package cn.com.qmth.examcloud.web.actuator;

import java.lang.reflect.Field;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

import org.springframework.boot.actuate.endpoint.annotation.Endpoint;
import org.springframework.boot.actuate.endpoint.annotation.ReadOperation;
import org.springframework.boot.actuate.endpoint.annotation.Selector;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.MediaType;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.commons.util.Calculator;
import cn.com.qmth.examcloud.commons.util.FreeMarkerUtil;
import cn.com.qmth.examcloud.commons.util.ResourceLoader;
import cn.com.qmth.examcloud.web.support.ApiInfo;
import cn.com.qmth.examcloud.web.support.ApiInfoHolder;

/**
 * 接口状态
 *
 * @author WANGWEI
 * @date 2019年7月25日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Configuration
@Endpoint(id = "api-status")
public class ApiStatusEndpoint {

	/**
	 * url: /actuator/api-status
	 *
	 * @author WANGWEI
	 * @return
	 */
	@ReadOperation(produces = {MediaType.TEXT_HTML_VALUE})
	public String getByDefault() {
		return get("");
	}

	/**
	 * url: /actuator/api-status/ignored?order=XX
	 *
	 * @author WANGWEI
	 * @param order
	 * @return
	 */
	@ReadOperation(produces = {MediaType.TEXT_HTML_VALUE})
	public String get(@Selector String order) {

		System.out.println("order by " + order);
		ReportorHolder.getApiDataReportor().report();
		List<ReportInfo> reportInfoList = ReportorHolder.getApiDataReportor().getReportInfoList();

		ReportInfo reportInfo = reportInfoList.get(0);
		List<MeterInfo> meterInfoList = reportInfo.getMeterInfoList();
		List<TimerInfo> timerInfoList = reportInfo.getTimerInfoList();

		Map<String, MeterInfo> meterInfoMap = Maps.newHashMap();

		for (MeterInfo meterInfo : meterInfoList) {
			String key = meterInfo.getKey();
			if (!key.startsWith(MetricNames.API_EXCEPTION_METER.name())) {
				continue;
			}
			String mapping = key.substring(MetricNames.API_EXCEPTION_METER.name().length() + 1);
			meterInfoMap.put(mapping, meterInfo);
		}

		List<ApiStatusInfo> list = Lists.newArrayList();

		for (TimerInfo timerInfo : timerInfoList) {
			String key = timerInfo.getKey();
			if (!key.startsWith(MetricNames.API_TIMER.name())) {
				continue;
			}

			String mapping = key.substring(MetricNames.API_TIMER.name().length() + 1);

			ApiStatusInfo apiStatusInfo = new ApiStatusInfo();
			list.add(apiStatusInfo);

			apiStatusInfo.setMapping(mapping);

			ApiInfo apiInfo = ApiInfoHolder.getApiInfo(mapping);
			if (null != apiInfo) {
				apiStatusInfo.setDescription(apiInfo.getDescription());
			}

			apiStatusInfo.setRateUnit(timerInfo.getRateUnit());
			apiStatusInfo.setDurationUnit(timerInfo.getDurationUnit());

			apiStatusInfo.setCount(timerInfo.getCount());
			apiStatusInfo.setMeanRate(timerInfo.getMeanRate());
			apiStatusInfo.setOneMinuteRate(timerInfo.getOneMinuteRate());
			apiStatusInfo.setFiveMinuteRate(timerInfo.getFiveMinuteRate());
			apiStatusInfo.setFifteenMinuteRate(timerInfo.getFifteenMinuteRate());

			apiStatusInfo.setMin(timerInfo.getMin());
			apiStatusInfo.setMax(timerInfo.getMax());
			apiStatusInfo.setMean(timerInfo.getMean());

			apiStatusInfo.setP50(timerInfo.getP50());
			apiStatusInfo.setP75(timerInfo.getP75());
			apiStatusInfo.setP95(timerInfo.getP95());
			apiStatusInfo.setP98(timerInfo.getP98());
			apiStatusInfo.setP99(timerInfo.getP99());
			apiStatusInfo.setP999(timerInfo.getP999());

			MeterInfo meterInfo = meterInfoMap.get(mapping);
			if (null == meterInfo) {
				apiStatusInfo.setExceptionCount(0L);
				apiStatusInfo.setExceptionMeanRate(0);
				apiStatusInfo.setExceptionOneMinuteRate(0);
				apiStatusInfo.setExceptionFiveMinuteRate(0);
				apiStatusInfo.setExceptionFifteenMinuteRate(0);

				apiStatusInfo.setExceptionPercent(0);
				apiStatusInfo.setExceptionMeanPercent(0);
				apiStatusInfo.setExceptionOneMinutePercent(0);
				apiStatusInfo.setExceptionFiveMinutePercent(0);
				apiStatusInfo.setExceptionFifteenMinutePercent(0);
			} else {
				apiStatusInfo.setExceptionCount(meterInfo.getCount());
				apiStatusInfo.setExceptionMeanRate(meterInfo.getMeanRate());
				apiStatusInfo.setExceptionOneMinuteRate(meterInfo.getOneMinuteRate());
				apiStatusInfo.setExceptionFiveMinuteRate(meterInfo.getFiveMinuteRate());
				apiStatusInfo.setExceptionFifteenMinuteRate(meterInfo.getFifteenMinuteRate());

				apiStatusInfo.setExceptionPercent(
						getPercent(meterInfo.getCount(), timerInfo.getCount()));
				apiStatusInfo.setExceptionMeanPercent(
						getPercent(meterInfo.getMeanRate(), timerInfo.getMeanRate()));
				apiStatusInfo.setExceptionOneMinutePercent(
						getPercent(meterInfo.getOneMinuteRate(), timerInfo.getOneMinuteRate()));
				apiStatusInfo.setExceptionFiveMinutePercent(
						getPercent(meterInfo.getFiveMinuteRate(), timerInfo.getFiveMinuteRate()));
				apiStatusInfo.setExceptionFifteenMinutePercent(getPercent(
						meterInfo.getFifteenMinuteRate(), timerInfo.getFifteenMinuteRate()));
			}

		}

		Collections.sort(list, new Comparator<ApiStatusInfo>() {
			@Override
			public int compare(ApiStatusInfo o1, ApiStatusInfo o2) {

				try {
					Field field = o1.getClass().getField(order);
					Object value1 = field.get(o1);
					Object value2 = field.get(o2);

					if (value1 instanceof Long) {
						return (int) (((Long) value1) - ((Long) value2));
					} else if (value1 instanceof Double) {
						return (int) (((Double) value1) - ((Double) value2));
					} else if (value1 instanceof String) {
						return ((String) value1).compareToIgnoreCase((String) value2);
					} else {
						return 0;
					}
				} catch (Exception e) {
					return 0;
				}
			}
		});

		Map<String, Object> map = Maps.newHashMap();
		map.put("data", list);
		map.put("dateTime", reportInfo.getDateTime());

		String html = ResourceLoader.getResource("api-status.html");

		String ret = FreeMarkerUtil.process(html, map);
		return ret;
	}

	private Double getPercent(double v1, double v2) {
		if (0 == v1 || 0 == v2) {
			return 0d;
		}
		double ret = Calculator.divide(v1, v2, 3);
		return ret;
	}

}
